@@ -20,7 +20,9 @@
 #include "Calculator.h"
 #include <limits.h>
 #include <stdlib.h>
-#include <sys/time.h>
+#ifndef _MSC_VER
+#	include <sys/time.h>
+#endif
 
 using std::string;
 using std::vector;
@@ -180,7 +182,7 @@ QalculateDateTime nextLeapSecond(const QalculateDateTime &date) {
 }
 QalculateDateTime prevLeapSecond(const QalculateDateTime &date) {
 	if(date.year() < LS_FIRST_YEAR) return QalculateDateTime();
-	size_t halfyear = sizeof(has_leap_second);
+	size_t halfyear = sizeof(has_leap_second) - 1;
 	if(date.year() <= LS_LAST_YEAR) {
 		halfyear = (date.year() - LS_FIRST_YEAR) * 2;
 		if(date.month() <= 6) {
@@ -494,6 +496,10 @@ bool QalculateDateTime::set(string str) {
 #ifndef _WIN32
 		}
 #endif
+	} else if(newyear <= 31 && newyear > 0 && newday > 999 && newmonth <= 12) {
+		long int y = newday;
+		newday = newyear;
+		newyear = y;
 	}
 	if(!set(newyear, newmonth, newday)) return false;
 	if(b_t) {
@@ -519,7 +525,16 @@ void QalculateDateTime::set(const QalculateDateTime &date) {
 	b_time = date.timeIsSet();
 }
 string QalculateDateTime::toISOString() const {
-	string str = i2s(i_year);
+	string str;
+	long int y = i_year;
+	if(y < 0) {
+		y = -y;
+		str += "-";
+	}
+	if(y < 10) str += "0";
+	if(y < 100) str += "0";
+	if(y < 1000) str += "0";
+	str += i2s(y);
 	str += "-";
 	if(i_month < 10) {
 		str += "0";
@@ -2725,3 +2740,37 @@ QalculateDateTime findNextLunarPhase(const QalculateDateTime &date, Number phase
 	return dt;
 }
 
+#ifdef _MSC_VER
+#	define DELTA_EPOCH_IN_MICROSECS  11644473600000000Ui64
+
+struct timezone {
+	int  tz_minuteswest; /* minutes W of Greenwich */
+	int  tz_dsttime;     /* type of dst correction */
+};
+
+int gettimeofday(struct timeval* tv, struct timezone* tz) {
+	FILETIME ft;
+	unsigned __int64 tmpres = 0;
+	static int tzflag = 0;
+	if(nullptr != tv) {
+		GetSystemTimeAsFileTime(&ft);
+		tmpres |= ft.dwHighDateTime;
+		tmpres <<= 32;
+		tmpres |= ft.dwLowDateTime;
+		tmpres /= 10;  /*convert into microseconds*/
+		/*converting file time to unix epoch*/
+		tmpres -= DELTA_EPOCH_IN_MICROSECS;
+		tv->tv_sec = (long)(tmpres / 1000000UL);
+		tv->tv_usec = (long)(tmpres % 1000000UL);
+	}
+	if(nullptr != tz) {
+		if(!tzflag) {
+			_tzset();
+			tzflag++;
+		}
+		tz->tz_minuteswest = _timezone / 60;
+		tz->tz_dsttime = _daylight;
+	}
+	return 0;
+}
+#endif
